"use strict";

const fs = require('fs');
const mkdirp = require('mkdirp');
const path = require('path');
const indexPage = require('./index-page.js');
const memberPage = require('./member-page.js');
const slug = require('./slug.js');
const utils = require('./utils.js');

const moduleFilter = (module) => module.children;

const kind = (member, module) => {
    let kind = member.kindString.toLowerCase();

    if (member.name.toUpperCase() === member.name) {
        kind = 'constant';
    } else if (module.name.match(/.component/) !== null) {
        kind = 'component';
    } else if (module.name.match(/.directive/) !== null) {
        kind = 'directive';
    } else if (member.type && member.type.type === 'union') {
        kind = 'union';
    } else {
        kind = 'class';
    }

    return kind;
};

const BUILTINS = {
    'Array': true, 'Date': true, 'Function': true,
    'TemplateRef': true, 'Observable': true
};

const assocChildren = (packageName, children) => {
    if (!children) {
        return;
    }

    children.forEach(child => {
        const type = child.type;
        if (type && type.type === 'reference' && !BUILTINS[type.name]) {
            child.typeSlug = slug(packageName, type.name);
        }
        assocChildren(packageName, child.children);
    });
};


const extractMembers = (packageName, modules) =>
    utils.flatten(modules
        .filter(moduleFilter)
        .map(module => module.children
            .map(child => {
                const model = {
                    id: child.id,
                    name: child.name,
                    comment: child.comment,
                    children: child.children,
                    decorators: child.decorators,
                    type: child.type,
                    kind: kind(child, module),
                    packageName: packageName,
                    url: `${child.name}`,
                    slug: slug(packageName, child.name)
                };

                assocChildren(packageName, model.children);
                return model;
            })
        )
    )
    .sort(utils.nameComparer);

const writeFile = (name, content) => {
    mkdirp.sync(path.dirname(name));
    fs.writeFileSync(name, content);
};

const writeMember = (path, member) =>
    writeFile(`${path}/${member.url}.md`, memberPage(member));

const generate = (done, config) => {
    const meta = require(config.jsonPath);
    const members = extractMembers(meta.name, meta.children);
    const outPath = config.outPath;

    writeFile(`${outPath}/index.md`, indexPage(meta.name, members));
    writeFile(`${outPath}/_meta.yml`, `---\ntitle: API\nposition: 2\n---\n`);
    members.forEach(member => writeMember(outPath, member));

    const warnings = utils.warnings();
    let error;
    if (config.warningsAsErrors && warnings.length > 0) {
        error = new Error(`API Generation failed with warnings`);
    }

    done(error);
};

module.exports = {
    generate: generate
};

